# -*- Perl -*-
#
# Licensed Materials - Property of IBM
# Rational ClearQuest
# (C) Copyright IBM Corp. 2009-2011.  All Rights Reserved.
# US Government Users Restricted Rights -
# Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
#
# File  : UtilLib.pm
#
# Description :
#   This perl module manages the logging of a script. It can log to a file or the console or both.
#   It prints the line number, then the error level, then the error itself.


package UtilLib;

my $module_name = "UtilLib";

use strict;
use warnings;
use File::Path;
use Time::localtime;
use Logger ();

# The singleton reference
my $util_singleton_reference = undef;

#*********************************
# PUBLIC functions
#*********************************

#
# Constructor
# 
# Arguments:
# @param 1 - self, the UtilLib object itself.
#
# Returns; 
# a reference to itself
#
sub new {
    my $class = shift;
    
    # Is the $singleton defined yet?
    if (! defined $util_singleton_reference) {
        
        my $self = {};
        # It's not so bless it
        $util_singleton_reference = bless $self, $class;

        # define module private data members
        $util_singleton_reference->{m_logger} = Logger->new();
    }

    # Return the reference
    return $util_singleton_reference;
}

#
# Call the command passed to it using the backtick method.
# If the command has a zero exit status, or DOS success level sub will return 1.
# If the command has a non-zero exit status, sub will return 0.
#
# This sub will return an exit status as well as the output from the command
# if the calling routine requests an array in return.
#
# Arguments:
# @param1 = the UtilLib object itself
# @param2 = the command to be executed.
#
# Returns:
# int: returns 0 for failure, 1 for success
#
sub ut_shell() {
    my $self = shift;   
    my $command = shift;
    
    my $function_name = "ut_shell";
    
    my $logger = Logger->new();
    
    my $output = "";
    my @return_value;

    $self->{m_logger}->log_print_message("$module_name: $function_name(): ִ");
    $self->{m_logger}->log_print_message("$module_name: $function_name(): $command");
    
    $output = `$command 2>&1`;

    if ($? == 0) {
        $self->{m_logger}->log_print_message("$module_name: $function_name(): ɹ$command");
        @return_value = (1, $output);
    } else {
        $self->{m_logger}->log_print_message("$module_name: $function_name(): ʧܣ" . $command);
        $self->{m_logger}->log_print_message("$module_name: $function_name(): ڵúĹԲ ");
        @return_value = (0, $output);
    }
    
    return wantarray ? @return_value : $return_value[0];
}

#
# Creates a directory
#
# Arguments:
# @param1 = the UtilLib object itself
# @param2 = the directory to be created
#
# Returns:
# nothing
#
sub ut_create_directory() {
    my $self = shift;
    my $dir = shift;
    
    my $function_name = "ut_create_directory";
    
    $self->{m_logger}->log_print_message("$module_name: $function_name(): ڴĿ¼$dir");
    
    if (! -e $dir) {
        mkdir $dir, 0777;
    }
}

#
# Creates a directory tree
#
# Arguments:
# @param1 = the UtilLib object itself
# @param2 = the directory tree to be created
#
# Returns:
# nothing
#
sub ut_create_directory_tree() {
    my $self = shift;
    my $dir = shift;
    
    my $function_name = "ut_create_directory_tree";

    $self->{m_logger}->log_print_message("$module_name: $function_name(): ڴĿ¼$dir");
    
    if (! -e $dir) {
        mkpath([$dir], 0, 0711);
    }
}

#
# Clears a directory
#
# Arguments:
# @param1 = the UtilLib object itself
# @param2 = the directory to be cleared
#
# Returns:
# nothing
#
sub ut_clear_directory() {
    my $self = shift;
    my $dir = shift;
    
    my $function_name = "ut_clear_directory";
    
    $self->{m_logger}->log_print_message("$module_name: $function_name(): Ŀ¼$dir");

    if (-e $dir) {
        $self->{m_logger}->log_print_message("$module_name: $function_name():  $dir");
        for my $file (glob("$dir/*")) {
            unlink($file);
        }
    }
    $self->ut_create_directory($dir);
}

#
# Clears a directory tree
#
# Arguments:
# @param1 = the UtilLib object itself
# @param2 = the directory tree to be cleared
#
# Returns:
# nothing
#
sub ut_clear_directory_tree() {
    my $self = shift;
    my $dir = shift;

    my $function_name = "ut_clear_directory_tree";
    
    $self->{m_logger}->log_print_message("$module_name: $function_name(): Ŀ¼$dir");
    if (-e $dir) {
        # catch warnings from rmtree and log them:
        local $SIG{__WARN__} = sub {
            $self->{m_logger}->log_print_error("$module_name: $function_name(): @_");
        };
        File::Path::rmtree($dir);
    }
    $self->ut_create_directory_tree($dir);
}

#
# Creates a time stamp suitable for logging purposes
#
# Arguments:
# @param1 = the UtilLib object itself
#
# Returns:
# string: the time stamp
#
sub ut_date_time_stamp() {
    my $self = shift;
    
    my $function_name = "ut_date_time_stamp";
    
    my $year = ( localtime->year() + 1900 ) % 2000;  # 2 digit year
    my $mon  = localtime->mon() + 1;
    my $mday = localtime->mday();
    my $hour = localtime->hour();
    my $min = localtime->min();
    my $sec = localtime->sec();
    
    if ( $year < 10 ) {
        $year = "0" . $year;
    }
    
    if ( $mon < 10 ) {
        $mon  = "0" . $mon;
    }
    
    if ( $mday < 10 ) {
        $mday = "0" . $mday;
    }

    if ( $hour < 10 ) {
        $hour = "0" . $hour;
    }

    if ( $min < 10 ) {
        $min  = "0" . $min;
    }

    if ( $sec < 10 ) {
        $sec = "0" . $sec;
    }
    
    my $date_str = $year . "-" . $mon.  "-" . $mday . ":" . $hour. ":" . $min . ":" . $sec;
    
    return $date_str;
}

#
# Creates a time stamp suitable for use as a file name
#
# Arguments:
# @param1 = the UtilLib object itself
#
# Returns:
# string: the time stamp
#
sub ut_file_name_date_time_stamp() {
    my $self = shift;
    
    my $function_name = "ut_file_name_date_time_stamp";
    
    my $year = ( localtime->year() + 1900 ) % 2000;  # 2 digit year
    my $mon  = localtime->mon() + 1;
    my $mday = localtime->mday();
    my $hour = localtime->hour();
    my $min = localtime->min();
    my $sec = localtime->sec();
    
    if ( $year < 10 ) {
        $year = "0" . $year;
    }
    
    if ( $mon < 10 ) {
        $mon  = "0" . $mon;
    }
    
    if ( $mday < 10 ) {
        $mday = "0" . $mday;
    }

    if ( $hour < 10 ) {
        $hour = "0" . $hour;
    }

    if ( $min < 10 ) {
        $min  = "0" . $min;
    }

    if ( $sec < 10 ) {
        $sec = "0" . $sec;
    }
    
    my $date_str = $year . $mon . $mday . "_" . $hour . $min . $sec;
    
    return $date_str;
}

#
# Returns a formated time only string, no date.
# 
# Arguments:
# @param 1 - self, the UtilLib object itself.
#
# Returns; 
# string: formated number representing time.
#
sub ut_short_time() {
    my $self = shift;

    my $function_name = "ut_short_time";

    my $sec = localtime->sec();
    my $min = localtime->min();
    my $hour = localtime->hour();

    if ( $hour < 10 ) {
        $hour = "0" . $hour;
    }

    if ( $min < 10 ) {
        $min  = "0" . $min;
    }

    if ( $sec < 10 ) {
        $sec = "0" . $sec;
    }

    return $hour. "." . $min . "." . $sec;
}

#
# Returns a the difference between two times
# 
# Arguments:
# @param 1 - self, the UtilLib object itself.
# @param 2 - The first time
# @param 3 - The second time
#
# Returns; 
# string: formated number representing time.
#
sub ut_diff_time() {
    my $self = shift;    
    my $time1 = shift;
    my $time2 = shift;
    
    my $function_name = "ut_diff_time";  
    
    if(!defined($time1) || !defined($time2)) {
        $self->{m_logger}->log_print_error("$module_name: $function_name(): ЧĲ time1$time1  time2$time2");
        return undef;
    }
    
    # Get the two times in seconds
    my $sec_time1 = $self->ut_prv_time_in_seconds_past_midnight($time1);
    my $sec_time2 = $self->ut_prv_time_in_seconds_past_midnight($time2);

    my $diff_sec;
    # if we've gone past midnight
    if ($sec_time1 > $sec_time2) {
        $diff_sec = 86400 - $sec_time1 + $sec_time2;
    } else {
        $diff_sec = $sec_time2 - $sec_time1;
    }

    return $self->ut_prv_convert_seconds_to_time_string($diff_sec);
}

#
# Creates a time stamp suitable for use in the database
#
# Arguments:
# @param1 = the UtilLib object itself
#
# Returns:
# string: the time stamp
#
sub ut_sql_date_time_stamp() {
    my $self = shift;
    
    my $function_name = "ut_sql_date_time_stamp";
    
    my $year = ( localtime->year() + 1900 ) % 2000;  # 2 digit year
    my $mon  = localtime->mon();
    my $mday = localtime->mday();
    my $hour = localtime->hour();
    my $min = localtime->min();
    my $sec = localtime->sec();
    
    if ( $year < 10 ) {
        $year = "0" . $year;
    }
    
    my $this_month = qw (Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec)[$mon];
    
    if ( $mday < 10 ) {
        $mday = "0" . $mday;
    }

    if ( $hour < 10 ) {
        $hour = "0" . $hour;
    }

    if ( $min < 10 ) {
        $min  = "0" . $min;
    }

    if ( $sec < 10 ) {
        $sec = "0" . $sec;
    }
    
    # It should look like this: 17-Oct-02.03:04:17
    
    my $date_str = $mday . "-" . $this_month . "-" . $year . " " . $hour . ":" . $min . ":" . $sec;
    
    return $date_str;
}

#
# replaces a string in a file
# 
# Arguments:
# @param 1 - self, the UtilLib object itself.
# @param 2 - the string to search for
# @param 3 - the string to replace it with
# @param 4 - the file name to change
#
# Returns; 
# int: returns 0 for failure, 1 for success
#
sub ut_replace_string () {
    my $self = shift;
    my $search_string = shift;
    my $replace_string = shift;
    my $file = shift;
    
    my $function_name = "ut_sql_date_time_stamp";
    
    if ( ( ! $search_string) || ( ! $replace_string ) || ( ! $file ) ) {
        $self->{m_logger}->log_print_error("$module_name: $function_name(): ȱ±֮һ");
        $self->{m_logger}->log_print_error("$module_name: $function_name(): ַ$search_string");
        $self->{m_logger}->log_print_error("$module_name: $function_name(): 滻ַ$replace_string");
        $self->{m_logger}->log_print_error("$module_name: $function_name(): ļ$file");
        return 0;
    }
                
    $self->{m_logger}->log_print_message("$module_name: $function_name(): Search$search_stringReplace$replace_stringFile$file");
    my $temp_file = "";
    
    if ( ! open(TEMPLATE, "< $file") ) {
        $self->{m_logger}->log_print_error("$module_name: $function_name(): ޷ļ$file");
        return 0;
    }
    
    while (<TEMPLATE>) {
        s/$search_string/$replace_string/e;
        $temp_file .= $_;
    }
    
    close(TEMPLATE);
    
    if ( ! open(WRITE, "> $file") ) {
        $self->{m_logger}->log_print_error("$module_name: $function_name(): ޷ļ $file Խд");
        return 0; 
    }
    
    print WRITE $temp_file;
    
    close(WRITE);
    
    return 1;
}

#*********************************
# PRIVATE functions
#*********************************

#
# Given a time stamp returns the time in seconds past midnight
#
# Arguments:
# @param1 = the UtilLib object itself
# @param2 = the time
#
# Returns:
# string: the time in seconds past midnight
#
sub ut_prv_time_in_seconds_past_midnight() {
    my $self = shift;
    my $time = shift;

    my $function_name = "ut_prv_time_in_seconds_past_midnight";  
    
    if(!defined($time)) {
        $self->{m_logger}->log_print_error("$module_name: $function_name(): ЧĲʱ䣺$time");
        return undef;
    }
    
    $time =~ /(..)(\.)(..)(\.)(..)/;
    my $hour = $1;
    my $min = $3;
    my $sec = $5;
    
    return $hour * 3600 + $min * 60 + $sec;
}

#
# Converts time in seconds past midnight to human readable time
#
# Arguments:
# @param1 = the UtilLib object itself
# @param2 = the time
#
# Returns:
# string: the human readable, elapsed time
#
sub ut_prv_convert_seconds_to_time_string() {
    my $self = shift;
    my $time = shift;

    my $function_name = "ut_prv_convert_seconds_to_time_string";
    
    if(!defined($time)) {
        $self->{m_logger}->log_print_error("$module_name: $function_name(): ЧĲʱ䣺$time");
        return undef;
    }
    
    my $hr = $time/3600;
    $hr =~ s/\..*//; # strip the remainder
    $time = $time % 3600;
    my $min = $time/60;
    $min =~ s/\..*//;  # strip the remainder
    my $sec = $time % 60;

    # Beautify the time
    if ($hr < 10) {
        $hr = "0" . $hr;
    }

    if ($min < 10) {
        $min = "0" . $min;
    }

    if ($sec < 10) {
        $sec = "0" . $sec;
    }

    return "$hr.$min.$sec";
}

1;
